@@ -1,7 +1,7 @@
 """
 The MIT License (MIT)
 
-Copyright (c) 2021 https://github.com/summer
+Copyright (c) 2020-present https://github.com/summer
 
 Permission is hereby granted, free of charge, to any person obtaining a
 copy of this software and associated documentation files (the "Software"),
@@ -21,6 +21,7 @@ LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 DEALINGS IN THE SOFTWARE.
 """
+
 import ast
 import base64
 import logging
@@ -30,29 +31,31 @@ from typing import Any, Dict, List, Optional
 import requests
 
 from .models import UserProfile
-
+from .http_client import HTTPClient
+from .exceptions import MojangError
 
 log = logging.getLogger(__name__)
 
 
-class MojangError(Exception):
-    """Base error class for all library-related exceptions in this file
-    Essentially, this could be caught to handle any exceptions thrown from this library.
-    """
+class API(HTTPClient):
 
-    def __init__(self, message: Optional[str] = None):
-        self.message = message if message else self.__class__.__doc__
-        super().__init__(self.message)
+    _API_BASE_URL = "https://api.mojang.com"
+    _SESSIONSERVER_BASE_URL = "https://sessionserver.mojang.com"
+    _AUTHSERVER_BASE_URL = "https://authserver.mojang.com"
 
+    def __init__(
+        self,
+        session: Optional[requests.Session] = None,
+        ratelimit_sleep_duration: Optional[int] = 60,
+        debug_mode: Optional[bool] = False,
+    ):
+        super().__init__(session, ratelimit_sleep_duration, debug_mode)
 
-class MojangAPI:
-
-    _API_DOMAIN = "https://api.mojang.com"
-    _SESSIONSERVER_DOMAIN = "https://sessionserver.mojang.com"
-    _AUTHSERVER_DOMAIN = "https://authserver.mojang.com"
-
-    @classmethod
-    def get_uuid(cls, username: str, timestamp: Optional[int] = None) -> Optional[str]:
+    def get_uuid(
+        self,
+        username: str,
+        timestamp: Optional[int] = None,
+    ) -> Optional[str]:
         """Convert a Minecraft name to a UUID.
 
         Warning: Limited Functionality
@@ -72,7 +75,7 @@ class MojangAPI:
 
         Example:
             ```py
-            uuid = MojangAPI.get_uuid("Notch")
+            uuid = api.get_uuid("Notch")
 
             if not uuid:
                 print("The username Notch is not taken")
@@ -82,15 +85,13 @@ class MojangAPI:
         """
 
         if timestamp:
-            url = (
-                cls._API_DOMAIN + f"/users/profiles/minecraft/{username}?at={timestamp}"
-            )
+            url = f"{self._API_BASE_URLf}/users/profiles/minecraft/{username}?at={timestamp}"
         else:
-            url = cls._API_DOMAIN + f"/users/profiles/minecraft/{username}"
+            url = f"{self._API_BASE_URL}/users/profiles/minecraft/{username}"
 
-        resp = requests.get(url)
+        resp = self.session.get(url)
 
-        if not resp.ok:
+        if resp.status_code == 400:
             return None
 
         try:
@@ -98,8 +99,7 @@ class MojangAPI:
         except json.decoder.JSONDecodeError:
             return None
 
-    @classmethod
-    def get_uuids(cls, names: List[str]) -> Dict[str, str]:
+    def get_uuids(self, names: List[str]) -> Dict[str, str]:
         """Convert up to 10 usernames to UUIDs in a single network request.
 
         Args:
@@ -114,28 +114,33 @@ class MojangAPI:
             ```py
             usernames = ["Notch", "Herobrine", "Dream"]
 
-            players = MojangAPI.get_uuids(usernames)
+            players = api.get_uuids(usernames)
 
             for name, uuid in players.items():
                 print(name, uuid)
             ```
         """
+        if not isinstance(names, list):
+            raise TypeError(
+                "Invalid data type passed. Make sure that you are passing a list of UUIDs instead of a string or dictionary."
+            )
+
         if len(names) > 10:
             names = names[:10]
 
-        resp = requests.post(cls._API_DOMAIN + "/profiles/minecraft", json=names)
-        data = resp.json()
+        data = self.session.post(
+            f"{self._API_BASE_URL}/profiles/minecraft", json=names
+        ).json()
 
         if not isinstance(data, list):
             if data.get("error"):
-                raise ValueError(data["errorMessage"])
+                raise MojangError(data["errorMessage"])
             else:
                 raise MojangError(data)
 
         return {name_data["name"]: name_data["id"] for name_data in data}
 
-    @classmethod
-    def get_username(cls, uuid: str) -> Optional[str]:
+    def get_username(self, uuid: str) -> Optional[str]:
         """Convert a UUID to a username.
 
         Args:
@@ -146,24 +151,25 @@ class MojangAPI:
 
         Example:
             ```py
-            username = MojangAPI.get_username("e149b689-d25c-4ace-a9ea-4be1e8407f85")
+            username = api.get_username("e149b689-d25c-4ace-a9ea-4be1e8407f85")
 
             if not username:
                 print("UUID does not appear to be valid.")
             ```
         """
-        resp = requests.get(
-            cls._SESSIONSERVER_DOMAIN + f"/session/minecraft/profile/{uuid}"
+        resp = self.session.get(
+            f"{self._SESSIONSERVER_BASE_URL}/session/minecraft/profile/{uuid}"
         )
-        if resp.ok:
-            try:
-                return resp.json()["name"]
-            except json.decoder.JSONDecodeError:
-                return None
-        return None
 
-    @classmethod
-    def get_profile(cls, uuid: str) -> Optional[UserProfile]:
+        if resp.status_code == 400:
+            return None
+
+        try:
+            return resp.json()["name"]
+        except json.decoder.JSONDecodeError:
+            return None
+
+    def get_profile(self, uuid: str) -> Optional[UserProfile]:
         """Get more information about a user from their UUID
 
         Args:
@@ -174,10 +180,10 @@ class MojangAPI:
 
         Example:
             ```py
-            uuid = MojangAPI.get_uuid("Notch")
+            uuid = api.get_uuid("Notch")
 
             if uuid:
-                profile = MojangAPI.get_profile(uuid)
+                profile = api.get_profile(uuid)
 
                 print(profile.name)
                 print(profile.skin_url)
@@ -185,8 +191,8 @@ class MojangAPI:
                 # is_legacy_profile, and timestamp
             ```
         """
-        resp = requests.get(
-            cls._SESSIONSERVER_DOMAIN + f"/session/minecraft/profile/{uuid}"
+        resp = self.session.get(
+            f"{self._SESSIONSERVER_BASE_URL}/session/minecraft/profile/{uuid}"
         )
 
         try:
@@ -196,8 +202,7 @@ class MojangAPI:
         user_profile = ast.literal_eval(base64.b64decode(value).decode())
         return UserProfile(user_profile)
 
-    @classmethod
-    def get_blocked_servers(cls) -> List[str]:
+    def get_blocked_servers(self) -> List[str]:
         """Get a list of SHA1 hashes of blacklisted Minecraft servers that do not follow EULA.
         These servers have to abide by the EULA or they will be shut down forever. The hashes are not cracked.
 
@@ -205,19 +210,18 @@ class MojangAPI:
             Blacklisted server hashes
 
         Example:
-        ```py
-        servers = MojangAPI.get_blocked_servers()
+            ```py
+            servers = api.get_blocked_servers()
 
-        for hash in servers:
-            print(hash)
-        ```
+            for hash in servers:
+                print(hash)
+            ```
         """
-        resp = requests.get(cls._SESSIONSERVER_DOMAIN + "/blockedservers")
+        resp = self.session.get(f"{self._SESSIONSERVER_BASE_URL}/blockedservers")
         return resp.text.splitlines()
 
-    @classmethod
     def refresh_access_token(
-        cls, access_token: str, client_token: str
+        self, access_token: str, client_token: str
     ) -> Dict[str, Any]:
         """Refreshes access token
 
@@ -233,7 +237,7 @@ class MojangAPI:
             access_token = "YOUR_ACCESS_TOKEN"
             client_token = "YOUR_CLIENT_TOKEN"
 
-            account = MojangAPI.refresh_access_token(access_token, client_token)
+            account = api.refresh_access_token(access_token, client_token)
 
             print("The new access token is " + account["access_token"])
 
@@ -255,7 +259,9 @@ class MojangAPI:
         }
 
         account = {}
-        data = requests.post(cls._AUTHSERVER_DOMAIN + "/refresh", json=payload).json()
+        data = self.session.post(
+            f"{self._AUTHSERVER_BASE_URL}/refresh", json=payload
+        ).json()
 
         account["username"] = data["user"]["username"]
         account["uuid"] = data["user"]["id"]
